/* eslint-disable no-console */
import type { HighlighterCore, LanguageRegistration, ThemeRegistrationAny } from '@shikijs/types'
import { createHighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import { createContext, useContext, useEffect, useMemo, useState } from 'react'

const DEBUG = true
function _log(...args: any[]) {
  if (DEBUG)
    console.log('[ShikiProvider]', ...args)
}

interface ShikiContextValue {
  highlighter: HighlighterCore | null
  isReady: boolean
  error: Error | null
}

interface ShikiProviderProps {
  children: React.ReactNode
  initialLanguages: LanguageRegistration[]
  initialThemes: ThemeRegistrationAny[]
}

const ShikiContext = createContext<ShikiContextValue | null>(null)

export function ShikiProvider({ children, initialLanguages, initialThemes }: ShikiProviderProps) {
  const [highlighter, setHighlighter] = useState<HighlighterCore | null>(null)
  const [isReady, setIsReady] = useState(false)
  const [error, setError] = useState<Error | null>(null)

  useEffect(() => {
    const initHighlighter = async () => {
      try {
        if (!initialThemes.length) {
          throw new Error('At least one theme must be provided')
        }

        _log(
          'Initializing with themes:',
          initialThemes.map(t => t.name),
        )

        const highlighterInstance = await createHighlighterCore({
          themes: initialThemes,
          langs: initialLanguages,
          engine: createJavaScriptRegexEngine({
            forgiving: true,
            cache: new Map(),
          }),
        })

        // Set the first theme as default
        const defaultTheme = initialThemes[0]
        if (defaultTheme) {
          highlighterInstance.setTheme(defaultTheme)
          _log('Default theme set:', defaultTheme.name)
        }

        _log('Initialization complete', {
          languages: highlighterInstance.getLoadedLanguages(),
          themes: highlighterInstance.getLoadedThemes(),
        })

        setHighlighter(highlighterInstance)
        setIsReady(true)
        setError(null)
      }
      catch (err) {
        console.error('Highlighter initialization failed:', err)
        setError(err instanceof Error ? err : new Error('Initialization failed'))
        setIsReady(false)
      }
    }

    initHighlighter()
  }, [initialLanguages, initialThemes])

  const value = useMemo<ShikiContextValue>(
    () => ({
      highlighter,
      isReady,
      error,
    }),
    [highlighter, isReady, error],
  )

  return <ShikiContext.Provider value={value}>{children}</ShikiContext.Provider>
}

export function useShiki() {
  const context = useContext(ShikiContext)
  if (!context) {
    throw new Error('useShiki must be used within a ShikiProvider')
  }
  return context
}
